package game.characters;

import game.GameCharacter;
import game.Game;

import java.util.List;
import java.util.Random;


/**
 * This class represents a Bee. Bee extends GameCharacter, which contains a GameCharacterImage, and
 * thus it can paint itself in a window.
 * @author hornick
 */
public class Bee extends GameCharacter {
	private final int RANDOM_INCREMENT = 3; // the maximum amount the Bee moves in each turn

	/**
	 * ctor - creates an instance of a Bee
	 * @param the bee's name
	 * @param imageFile image to display
	 * @param soundFile sound to make
	 */
	public Bee(String name, String imageFile, String soundFile ) {
		super( name, imageFile, soundFile ); // make the GameCharacter ctor do the work...
	}

	/**
	 * This method is called by the Game framework every turn to make the Bee plan its next move.
	 * This Bee is pretty stupid - each move is a random increment to a new row and column.
	 * 
	 * This method must be overridden by the specific GameCharacter-derived classes, which
	 * determine exactly what to do in each turn. Note that the derived class's implementation
	 * of this method must include a call to validateMove() so that the game framework can
	 * determine whether the move is legal.
	 * @param list a List of GameCharacters that are on the board, which can be used for planning purposes
	 * This Bee does nothing with information available from the list.
	 */
	@Override
	public void planMove(List<GameCharacter> list) {
		int deltaR; // incremental movement value in row direction
		int deltaC; // same, but in column direction
		Random r = new Random();
		deltaR = r.nextInt(2*RANDOM_INCREMENT+1)-RANDOM_INCREMENT;	// generate random values between -INCREMENT and INCREMENT
		deltaC = r.nextInt(2*RANDOM_INCREMENT+1)-RANDOM_INCREMENT;  	//  to determine how far the Bee will move this turn

		int rowNext = getCurrentRow() + deltaR;		// the new target row and column to move to
		int colNext = getCurrentColumn() + deltaC;

		setTargetPosition(rowNext, colNext); // set the new target position; the Game framework will do the actual move
		Game.validateMove(this);	// This MUST be called for the Game framework to validate the target position
	}

	/**
	 * This method is called by the Game framework at the end of every move to give the GameCharacter
	 * the chance to take some action.
	 * @param list a list of GameCharacters in the same board cell as this GameCharacter
	 */
	@Override
	public void finishMove(List<GameCharacter> list) {
		for( GameCharacter neighbor: list ) { // iterate through each neighboring GameCharacter in the same cell
			int points = Game.tag(this, neighbor);	// try to tag it in order to score points
			
			if(points==0)					// if no points were obtained, buzz!
				makeSound();
		}
	}
}
